﻿Public Class Form1

    Private Sub btnSimulation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSimulation.Click
        'Simulation procedure (loop)
        'Main loop of simulation
        Dim maxSimTime As Double = Val(txtSimTime.Text)
        Dim myTruck As New TruckEntity
        Dim myPlant As New AsphaltPlant
        Dim myEngine As New Engine
        Dim myFELItem As New FELItem
        'Initialize simulation by scheduling first truck arrival
        initializeSim(myEngine, myPlant)
        While myEngine.curTime < maxSimTime
            myFELItem = myEngine.nextEvent()
            myPlant.eventHappened(myEngine, myFELItem.entity, myFELItem.simEvent)
            txtCurTime.Text = myEngine.curTime.ToString
            txtAvgQL.Text = myPlant.avgQLength.ToString
            txtAvgWaitT.Text = myPlant.avgQWaitingTime.ToString
            txtUtilization.Text = myPlant.avgUtilization.ToString
            txtTruckServed.Text = myPlant.totalTruckServred.ToString
            Me.Refresh()
        End While

    End Sub


    'Initialization Procedure
    Public Sub initializeSim(ByRef myEngine As Engine, ByRef myPlant As AsphaltPlant)
        Dim myFirstFELItem As New FELItem
        myFirstFELItem.entity.truckId = 1
        myFirstFELItem.eventTime = myPlant.nextTruckArrival
        myFirstFELItem.simEvent = eventType.TruckArrival
        myEngine.scheduleNewEvent(myFirstFELItem)
    End Sub

End Class

Public Class TruckEntity
    Public truckId As Integer
    Public enterQTime As Double
End Class

'Enumerations are integer types making the editor handy using a textual entry format
Public Enum eventType
    TruckArrival = 1
    LoadingCompletion = 2
End Enum

Public Class FELItem
    Public simEvent As eventType
    Public entity As New TruckEntity
    Public eventTime As Double
End Class

Public Class Engine
    Public FEL As New Collection
    Public curTime As Double = 0
    Public Sub scheduleNewEvent(ByVal newFELItem As FELItem)
        FEL.Add(newFELItem, newFELItem.entity.truckId.ToString)
    End Sub

    Public Function nextEvent() As FELItem
        nextEvent = New FELItem
        nextEvent.entity.truckId = 0
        nextEvent.eventTime = Double.MaxValue
        For Each myFELItem As FELItem In FEL
            If myFELItem.eventTime < nextEvent.eventTime Then
                nextEvent.entity = myFELItem.entity
                nextEvent.eventTime = myFELItem.eventTime
                nextEvent.simEvent = myFELItem.simEvent
            End If
        Next
        curTime = nextEvent.eventTime
        If nextEvent.entity.truckId > 0 Then
            FEL.Remove(nextEvent.entity.truckId.ToString)
        End If
    End Function
End Class
'Using Enumeration
Public Enum resourceStatus
    Idle = 0
    Busy = 1
End Enum

Public Class AsphaltPlant
    'Properties
    Public status As resourceStatus
    Public Q As New Collection

    Public avgQLength As Double = 0
    Public avgQWaitingTime As Double = 0
    Public avgUtilization As Double = 0
    Public totalTruckServred As Integer = 0

    Public totalTruckPassedQ As Integer = 0
    Public lastStatusReportTime As Double = 0
    Public totalTruckArrived As Integer = 0

    Public Event newTruckArrived(ByRef myEngine As Engine, ByRef newTruck As TruckEntity)
    Public Event loadingCompleted(ByRef myEngine As Engine)

    Public Function eventHappened(ByRef myEngine As Engine, ByRef newTruck As TruckEntity, ByVal myEvent As eventType) As eventType
        eventHappened = myEvent
        Select Case eventHappened
            Case eventType.TruckArrival
                RaiseEvent newTruckArrived(myEngine, newTruck)
            Case eventType.LoadingCompletion
                RaiseEvent loadingCompleted(myEngine)
        End Select
    End Function

    Public Function loadingTime() As Double
        Dim z As Double
        z = Math.Pow(-2 * Math.Log(Rnd()), 0.5) * Math.Cos(2 * Math.PI * Rnd())
        loadingTime = 10 + z * 2
    End Function

    Public Function nextTruckArrival() As Double
        nextTruckArrival = -15 * Math.Log(Rnd())
    End Function

    Public Function firstTruckInQToLoading() As TruckEntity
        firstTruckInQToLoading = New TruckEntity
        firstTruckInQToLoading.enterQTime = Double.MaxValue
        For Each myTruckItem As TruckEntity In Q
            If myTruckItem.enterQTime < firstTruckInQToLoading.enterQTime Then
                firstTruckInQToLoading.enterQTime = myTruckItem.enterQTime
                firstTruckInQToLoading.truckId = myTruckItem.truckId
            End If
        Next
        Q.Remove(firstTruckInQToLoading.truckId.ToString)
    End Function

    Public Sub _newTruckArrived(ByRef myEngine As Engine, ByRef newTruck As TruckEntity) Handles Me.newTruckArrived
        '1) Schedule next truck arrival event!
        Dim newFELItem As New FELItem
        totalTruckArrived = totalTruckArrived + 1
        newFELItem.entity.truckId = totalTruckArrived + 1
        newFELItem.eventTime = myEngine.curTime + nextTruckArrival()
        newFELItem.simEvent = eventType.TruckArrival
        myEngine.scheduleNewEvent(newFELItem)
        '2)Handle Truck arrival in plant if plant is busy
        If status = resourceStatus.Busy Then 'prepare and send truck to the Q
            'calculate utilization
            avgUtilization = (avgUtilization * lastStatusReportTime + (myEngine.curTime - lastStatusReportTime)) / myEngine.curTime
            'calculate avg Q length
            avgQLength = (avgQLength * lastStatusReportTime + Q.Count * (myEngine.curTime - lastStatusReportTime)) / myEngine.curTime
            'add truck to the Q
            newTruck.enterQTime = myEngine.curTime
            Q.Add(newTruck, newTruck.truckId.ToString)

        Else '3) if plants status is idle send the truck to the loading
            'calculate avg waitting time considering current truck's waiting time as 0
            avgQWaitingTime = avgQWaitingTime * totalTruckPassedQ / (totalTruckPassedQ + 1)
            'calculate utilization 
            avgUtilization = avgUtilization * lastStatusReportTime / myEngine.curTime
            'calculate avg Q length
            avgQLength = avgQLength * lastStatusReportTime / myEngine.curTime
            'Change Status to busy
            status = resourceStatus.Busy
            'Increase number of trucks passed the Q
            totalTruckPassedQ = totalTruckPassedQ + 1
            'Schedule loading completion event
            newFELItem = New FELItem
            newFELItem.entity.truckId = newTruck.truckId
            newFELItem.eventTime = myEngine.curTime + loadingTime()
            newFELItem.simEvent = eventType.LoadingCompletion
            myEngine.scheduleNewEvent(newFELItem)
        End If
        lastStatusReportTime = myEngine.curTime
    End Sub

    Public Sub _loadingCompleted(ByRef myEngine As Engine) Handles Me.loadingCompleted
        totalTruckServred = totalTruckServred + 1
        'calculate utilization
        avgUtilization = (avgUtilization * lastStatusReportTime + (myEngine.curTime - lastStatusReportTime)) / myEngine.curTime
        '1) If there is no truck waiting to be loaded, make asphalt plant idle
        If Q.Count = 0 Then
            'calculate avg Q length
            avgQLength = avgQLength * lastStatusReportTime / myEngine.curTime
            'Change Status to idle
            status = resourceStatus.Idle

        Else '2)If trucks waiting in the Q, number of trucks waiting to be loaded is decreased by one; Schedule a new  loading completion event!
            'calculate avg Q length
            avgQLength = (avgQLength * lastStatusReportTime + Q.Count * (myEngine.curTime - lastStatusReportTime)) / myEngine.curTime
            'Retrieve first truck from the Q to be loaded
            Dim myTruckToLoading As TruckEntity = firstTruckInQToLoading()
            'calculate avg waitting time
            avgQWaitingTime = (avgQWaitingTime * totalTruckPassedQ + myEngine.curTime - myTruckToLoading.enterQTime) / (totalTruckPassedQ + 1)
            'Increase number of trucks passed the Q
            totalTruckPassedQ = totalTruckPassedQ + 1
            Dim newFELItem As New FELItem
            newFELItem.entity.truckId = myTruckToLoading.truckId
            newFELItem.eventTime = myEngine.curTime + loadingTime()
            newFELItem.simEvent = eventType.LoadingCompletion
            myEngine.scheduleNewEvent(newFELItem)
        End If
        lastStatusReportTime = myEngine.curTime
    End Sub




End Class